#include "audiowidget.h"
#include <cmath>

AudioWidget::AudioWidget(QWidget *parent) :
    QWidget(parent)
  , m_layout(new QHBoxLayout())
{
    for (int i= 0; i < Settings::getInstance()->getLayersNumber(); i++ ) {
        AudioLayerWidget *alw = new AudioLayerWidget(this, i);
        m_layout->insertWidget(i, alw);
        connect(alw, SIGNAL(uiSliderChanged(int, Slider, int)), this, SIGNAL(uiSliderChanged(int, Slider, int)));
        connect(alw, SIGNAL(uiPlaybackChanged(int, Status)), this, SIGNAL(uiPlaybackChanged(int, Status)));
        connect(alw, SIGNAL(uiLoadMedia(int, QString)), this, SIGNAL(uiLoadMedia(int, QString)));
    }
    m_layout->setSpacing(0);
    m_layout->setContentsMargins(1, 1, 1, 1);
    setLayout(m_layout);
    m_refreshUi = new QTimer(this);
    connect(m_refreshUi, SIGNAL(timeout()), this, SLOT(refreshUi()));
    m_refreshUi->start(UI_REFRESH_TIME * 2);
}

void AudioWidget::mediaLoaded(int layer, QString file, float duration)
{
    m_layerUpdate[layer].media = file;
    m_layerUpdate[layer].duration = duration;
    m_layerUpdate[layer].updated = true;
}

void AudioWidget::volChanged(int layer, float vol) {
    m_layerUpdate[layer].vol = vol;
    m_layerUpdate[layer].updated = true;
}

void AudioWidget::panChanged(int layer, int pan) {
    m_layerUpdate[layer].pan = pan;
    m_layerUpdate[layer].updated = true;
}

void AudioWidget::pitchChanged(int layer, int pitch) {

    m_layerUpdate[layer].pitch = pitch;
    m_layerUpdate[layer].updated = true;
}

void AudioWidget::playbackChanged(int layer, Status status)
{
    m_layerUpdate[layer].status = status;
    m_layerUpdate[layer].updated = true;
}

void AudioWidget::cursorChanged(int layer, float cursor)
{
    m_layerUpdate[layer].cursor = floor(cursor * 1000) / 1000;
    m_layerUpdate[layer].updated = true;
}

void AudioWidget::refreshUi()
{
    for (int i = 0; i < MAX_LAYERS; i++)
    {
        if (m_layerUpdate[i].updated) {
            QLayoutItem * const item = m_layout->itemAt(i);
            AudioLayerWidget *alw = dynamic_cast<AudioLayerWidget *>(item->widget());
            if (m_layerUpdate[i].vol > -1) {
                alw->setVol(m_layerUpdate[i].vol);
                m_layerUpdate[i].vol = -1;
            }
            if (m_layerUpdate[i].cursor > -1) {
                alw->setCurrentTime(m_layerUpdate[i].cursor);
                m_layerUpdate[i].cursor = -1;
            }
            if (m_layerUpdate[i].pan > -1) {
                alw->setPan(m_layerUpdate[i].pan);
                m_layerUpdate[i].pan = -1;
            }
            if (m_layerUpdate[i].pitch > -1) {
                alw->setPitch(m_layerUpdate[i].pitch);
                m_layerUpdate[i].pitch = -1;
            }
            if (m_layerUpdate[i].status != Status::Iddle) {
                alw->setPlaybackStatus(m_layerUpdate[i].status);
                m_layerUpdate[i].status = Status::Iddle;
            }
            if (m_layerUpdate[i].duration > -1) {
                alw->setMediaFile(m_layerUpdate[i].media);
                alw->setDuration(m_layerUpdate[i].duration);
                m_layerUpdate[i].duration = -1;
            }
            m_layerUpdate[i].updated = false;
        }
    }
}
